@@ -28,11 +28,12 @@ enum as_filter_type
 };
 
 extern void bgp_filter_init (void);
+extern void bgp_filter_reset (void);
 
 extern enum as_filter_type as_list_apply (struct as_list *, void *);
 
 extern struct as_list *as_list_lookup (const char *);
-extern void as_list_add_hook (void (*func) ());
-extern void as_list_delete_hook (void (*func) ());
+extern void as_list_add_hook (void (*func) (void));
+extern void as_list_delete_hook (void (*func) (void));
 
 #endif /* _QUAGGA_BGP_FILTER_H */
