@@ -110,7 +110,7 @@ static struct prefix_master prefix_master_orf =
   NULL,
   NULL,
 };
-
+
 static struct prefix_master *
 prefix_master_get (afi_t afi)
 {
@@ -621,7 +621,7 @@ prefix_list_print (struct prefix_list *plist)
 	}
     }
 }
-
+
 /* Retrun 1 when plist already include pentry policy. */
 static struct prefix_list_entry *
 prefix_entry_dup_check (struct prefix_list *plist,
@@ -733,16 +733,16 @@ vty_prefix_list_install (struct vty *vty, afi_t afi, const char *name,
 #endif /* HAVE_IPV6 */
 
   /* ge and le check. */
-  if (genum && genum <= p.prefixlen)
+  if (genum && (genum <= p.prefixlen))
     return vty_invalid_prefix_range (vty, prefix);
 
-  if (lenum && lenum <= p.prefixlen)
+  if (lenum && (lenum <= p.prefixlen))
     return vty_invalid_prefix_range (vty, prefix);
 
-  if (lenum && genum > lenum)
+  if (lenum && (genum > lenum))
     return vty_invalid_prefix_range (vty, prefix);
 
-  if (genum && lenum == (afi == AFI_IP ? 32 : 128))
+  if (genum && (lenum == (afi == AFI_IP ? 32 : 128)))
     lenum = 0;
 
   /* Get prefix_list with name. */
@@ -1165,7 +1165,7 @@ vty_clear_prefix_list (struct vty *vty, afi_t afi, const char *name,
     }
   return CMD_SUCCESS;
 }
-
+
 DEFUN (ip_prefix_list,
        ip_prefix_list_cmd,
        "ip prefix-list WORD (deny|permit) (A.B.C.D/M|any)",
@@ -1759,7 +1759,7 @@ DEFUN (clear_ip_prefix_list_name_prefix,
 {
   return vty_clear_prefix_list (vty, AFI_IP, argv[0], argv[1]);
 }
-
+
 #ifdef HAVE_IPV6
 DEFUN (ipv6_prefix_list,
        ipv6_prefix_list_cmd,
@@ -2355,7 +2355,7 @@ DEFUN (clear_ipv6_prefix_list_name_prefix,
   return vty_clear_prefix_list (vty, AFI_IP6, argv[0], argv[1]);
 }
 #endif /* HAVE_IPV6 */
-
+
 /* Configuration write function. */
 static int
 config_write_prefix_afi (afi_t afi, struct vty *vty)
@@ -2623,7 +2623,7 @@ prefix_list_reset_orf (void)
 
 
 /* Prefix-list node. */
-struct cmd_node prefix_node =
+static struct cmd_node prefix_node =
 {
   PREFIX_NODE,
   "",				/* Prefix list has no interface. */
@@ -2732,7 +2732,7 @@ prefix_list_init_ipv4 (void)
 
 #ifdef HAVE_IPV6
 /* Prefix-list node. */
-struct cmd_node prefix_ipv6_node =
+static struct cmd_node prefix_ipv6_node =
 {
   PREFIX_IPV6_NODE,
   "",				/* Prefix list has no interface. */
