@@ -28,7 +28,7 @@ Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 #include "command.h"
 #include "vty.h"
 #include "log.h"
-
+
 /* Vector for route match rules. */
 static vector route_match_vec;
 
@@ -72,7 +72,7 @@ route_map_rule_delete (struct route_map_rule_list *,
 
 static void
 route_map_index_delete (struct route_map_index *, int);
-
+
 /* New route map allocation. Please note route map's name must be
    specified. */
 static struct route_map *
@@ -420,7 +420,7 @@ route_map_rule_new (void)
   new = XCALLOC (MTYPE_ROUTE_MAP_RULE, sizeof (struct route_map_rule));
   return new;
 }
-
+
 /* Install rule command to the match list. */
 void
 route_map_install_match (struct route_map_rule_cmd *cmd)
@@ -889,7 +889,16 @@ route_map_init (void)
   route_match_vec = vector_init (1);
   route_set_vec = vector_init (1);
 }
-
+
+void
+route_map_finish (void)
+{
+  vector_free (route_match_vec);
+  route_match_vec = NULL;
+  vector_free (route_set_vec);
+  route_set_vec = NULL;
+}
+
 /* VTY related functions. */
 DEFUN (route_map,
        route_map_cmd,
@@ -1280,7 +1289,7 @@ route_map_config_write (struct vty *vty)
 }
 
 /* Route map node structure. */
-struct cmd_node rmap_node =
+static struct cmd_node rmap_node =
 {
   RMAP_NODE,
   "%s(config-route-map)# ",
