@@ -59,6 +59,7 @@
 #include "zebra/irdp.h"
 #include <netinet/ip_icmp.h>
 #include "if.h"
+#include "checksum.h"
 #include "sockunion.h"
 #include "log.h"
 #include "sockopt.h"
@@ -67,12 +68,9 @@
 /* GLOBAL VARS */
 
 int irdp_sock = -1;
-char b1[16], b2[16], b3[16], b4[16];  /* For inet_2a */
 
 extern struct zebra_t zebrad;
 extern struct thread *t_irdp_raw;
-int in_cksum (void *ptr, int nbytes);
-void process_solicit (struct interface *ifp);
 
 static void
 parse_irdp_packet(char *p, 
@@ -186,7 +184,7 @@ parse_irdp_packet(char *p,
 		 inet_ntoa (src));
     }
 }
-
+
 static int
 irdp_recvmsg (int sock, u_char *buf, int size, int *ifindex)
 {
@@ -224,14 +222,14 @@ irdp_recvmsg (int sock, u_char *buf, int size, int *ifindex)
 
   return ret;
 }
-
+
 int irdp_read_raw(struct thread *r)
 {
   struct interface *ifp;
   struct zebra_if *zi;
   struct irdp_interface *irdp;
   char buf[IRDP_RX_BUF];
-  int ret, ifindex;
+  int ret, ifindex = 0;
   
   int irdp_sock = THREAD_FD (r);
   t_irdp_raw = thread_add_read (zebrad.master, irdp_read_raw, NULL, irdp_sock);
@@ -266,7 +264,7 @@ int irdp_read_raw(struct thread *r)
 
   return ret;
 }
-
+
 void 
 send_packet(struct interface *ifp, 
 	    struct stream *s,
@@ -289,7 +287,7 @@ send_packet(struct interface *ifp,
   if (!(ifp->flags & IFF_UP))
     return;
 
-  if (!p) 
+  if (p)
     src = ntohl(p->u.prefix4.s_addr);
   else 
     src = 0; /* Is filled in */
